#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Feb 12 09:42:30 2021

@author: lbresin
"""
import pandas as pd
import geopandas as gpd
import numpy as np
import os

def write_hgs_instruction(file_dst, df_DP, simulation_start, bc_start, bc_end, bc_info):
    df_DP_write = df_DP[(df_DP.index >= bc_start) & (df_DP.index <= bc_end)]
    with open(file_dst, 'w') as fid:
        fid.write('!---- transient deep percolation\n')
        fid.write('!-- Info: \t\t{}\n'.format(bc_info))
        fid.write('!-- Unit: \t\tm d⁻¹\n')
        fid.write('!-- Simulation Start: \t{}\n'.format(simulation_start.strftime('%Y-%m-%d')))
        fid.write('!-- BC Start: \t\t{} (i.e., {} sim. days)\n'.format(bc_start.strftime('%Y-%m-%d'),(bc_start - simulation_start).days))
        fid.write('!-- BC End: \t\t{} (i.e., {} sim. days)\n\n'.format(bc_end.strftime('%Y-%m-%d'),(bc_end - simulation_start).days))
        fid.write('clear chosen elements\nclear chosen zones\nclear chosen faces\nclear chosen nodes\n\n')
        
        for zone in df_DP_write.columns:
            fid.write('!---- Zone {a:} - {a:}\n'.format(a=zone))
            fid.write('choose faces top from raster\n  ./hgs_files/area_selection_files/dp_subbasins.asc\n  {0:} {0:}\n\n'.format(zone))
            fid.write('write chosen faces\n  faces_dp_zone_{0:}_top.txt\n\n'.format(zone))
            fid.write('create face set from chosen faces\n  top_dp_zone{0:}\n\n'.format(zone))
            fid.write('boundary condition\n  type\n  rain\n\n  face set\n  top_dp_zone{0:}\n\n'.format(zone))
            fid.write('  time value table\n')
            if simulation_start < bc_start:
                fid.write('  {a:5d} {b:}\n'.format(a=0, b='-99999.'))
            elif simulation_start > bc_start:
                print('error bc starts earlier than simulation start')
                os.sys.exit()
            rch_height = 1
            for i in range(df_DP_write.shape[0]):
                date = df_DP_write.index[i]
                rch_height = df_DP_write.loc[date, zone]
                if (i == 0) | (i == df_DP_write.shape[0] - 1):  # FIRST or LAST LINE
                    fid.write('  {a:5d} {b:1.4e}\n'.format(a=(date-simulation_start).days, b=rch_height))
                else: # ALL OTHER LINES, but first and last line
                    date_prev = df_DP_write.index[i-1]
                    rch_height_prev = df_DP_write.loc[date_prev, zone]
                    date_next= df_DP_write.index[i+1]
                    rch_height_next = df_DP_write.loc[date_next, zone]
                    if (rch_height != 0) | (rch_height_prev != 0) | (rch_height_next != 0):
                        fid.write('  {a:5d} {b:1.4e}\n'.format(a=(date-simulation_start).days, b=rch_height))
                    else:
                        pass
            fid.write('  {a:5d} {b:}\n'.format(a=(bc_end-simulation_start).days+1, b='-99999.'))
            fid.write('  end\n')
            fid.write('  interpolate\n  tecplot output\nend !new specified head\n\n')
            fid.write('clear chosen elements\nclear chosen zones\nclear chosen faces\nclear chosen nodes\n\n')

def write_hgs_instruction_mean(file_dst, df_DP, bc_info):
    df_DP_write = df_DP.mean()
    with open(file_dst, 'w') as fid:
        fid.write('!---- Mean deep percolation (Predevelopment)\n')
        fid.write('!-- Info: \t\t{}\n'.format(bc_info))
        fid.write('!-- Unit: \t\tm d⁻¹\n\n')
        fid.write('clear chosen elements\nclear chosen zones\nclear chosen faces\nclear chosen nodes\n\n')
        
        for zone in df_DP_write.index:
            fid.write('!---- Zone {a:} - {a:}\n'.format(a=zone))
            fid.write('choose faces top from raster\n  ./hgs_files/area_selection_files/dp_subbasins.asc\n  {0:} {0:}\n\n'.format(zone))
            fid.write('write chosen faces\n  faces_dp_zone_{0:}_top.txt\n\n'.format(zone))
            fid.write('create face set from chosen faces\n  top_dp_zone{0:}\n\n'.format(zone))
            fid.write('boundary condition\n  type\n  rain\n\n  face set\n  top_dp_zone{0:}\n\n'.format(zone))
            fid.write('  time value table\n')
 
            rch_height = df_DP_write[zone]
            fid.write('  0 {b:1.4e}\n'.format(b=rch_height))
            fid.write('  end\n')
            fid.write('end !new specified head\n\n')
            fid.write('clear chosen elements\nclear chosen zones\nclear chosen faces\nclear chosen nodes\n\n')



if __name__ == '__main__':
    simulation_start = pd.to_datetime('1950-09-15')
    
    ### OPEN RELEVANT DATA ###
    ## RCH Subbasins ##
    sub_shapefile = gpd.read_file('./shape_files/rch_zones.shp').set_index('Subbasin')
    sub_shapefile.index = sub_shapefile.index.astype(int) 
    sub_shapefile.ID = sub_shapefile.ID.astype(int)
    
    ## Soil Water Balance Model CLIMATE MODEL ##
    df_DP_schmidt = pd.read_csv('./SEWBM/lbresin_climate_model_1981-2070_m_d-1.csv', index_col=0) # m/d
    df_DP_schmidt.columns = df_DP_schmidt.columns.astype(int)
    df_DP_schmidt.index = [pd.Timestamp(i) for i in df_DP_schmidt.index]
    df_DP_schmidt = df_DP_schmidt.reindex(sorted(df_DP_schmidt.columns), axis=1)
    
    ## Soil Water Balance Model ##
    df_DP_schmidt_IMS = pd.read_csv('./SEWBM/lbresin_ims_modis_dp_1990-2018_m_d-1.csv', index_col=0) # m/d
    df_DP_schmidt_IMS.columns = df_DP_schmidt_IMS.columns.astype(int)
    df_DP_schmidt_IMS.index = [pd.Timestamp(i) for i in df_DP_schmidt_IMS.index]
    df_DP_schmidt_IMS = df_DP_schmidt_IMS.reindex(sorted(df_DP_schmidt_IMS.columns), axis=1)
    
    ## Cutoff data prior Scenario begin
    scenario_start_date = pd.Timestamp('2019-09-15')
    scenario_end_date = pd.Timestamp('2070-12-31')
    df_DP_export = df_DP_schmidt[(df_DP_schmidt.index >= scenario_start_date) &
                                 (df_DP_schmidt.index <= scenario_end_date)]
    
   
    ### WRITE HGS INPUT ###   
    file_dst = './export/deep_percolation_3x3km_climate_model_2021-2050.in'
    bc_start = df_DP_export.index.min()
    bc_end = df_DP_export.index.max()
    bc_info = 'Deep percolation computed via the SWBM Schmidt-Bresinsky (from 2021 to 2050 in daily timesteps).'
    write_hgs_instruction(file_dst, df_DP_export, simulation_start, bc_start, bc_end, bc_info)

    

    ### PLOT ###
    import matplotlib.pyplot as plt
    import matplotlib
    matplotlib.rcParams['figure.dpi'] = 300
    
    # Plot 1    
    fig = plt.figure(figsize=(20/2.5, 7/2.5))
    ax = plt.gca()

    ax.plot(df_DP_export.sum(axis=1))
    
    fig.savefig('./plot/timeserries.png')

    ax.set_xlim([pd.Timestamp('2021-01-01'), pd.Timestamp('2050-12-31')])  
    
    fig.savefig('./plot/timeserries_2021_2050.png')
    
    fig.clf()
    
    # Plot 2    
    
    fig = plt.figure(figsize=(20/2.5, 7/2.5))
    ax = plt.gca()
    
    tmp = df_DP_export.sum(axis=1)
    tmp = tmp.groupby(tmp.index.year).sum()
    ax.bar(tmp.index, tmp, width=0.9)
    
    fig.savefig('./plot/timeserries_annual.png')

    #ax.set_xlim([1985.5, 1993.5])  
    
    fig.savefig('./plot/timeserries_annual_2021-2050.png')
    
    fig.clf()
    
    # Plot 3
    
    fig = plt.figure(figsize=(20/2.5, 7/2.5))
    ax = plt.gca()
    
    tmp = df_DP_schmidt.sum(axis=1).groupby(df_DP_schmidt.index.year).sum()
    tmp2 = df_DP_schmidt_IMS.sum(axis=1).groupby(df_DP_schmidt_IMS.index.year).sum()

    
    width = 0.4
    n = 2
    
    ax.bar(tmp.index - (width/2) + (width/n) * 0.5, tmp, width=width/2)
    ax.bar(tmp2.index - (width/2) + (width/n) * 1.5, tmp2, width=width/2)
    
    fig.savefig('./plot/timeserries_annual_comp.png')

    #ax.set_xlim([1985.5, 1993.5])  
    
    fig.savefig('./plot/timeserries_annual_comp_2021-2050.png')
    
    fig.clf()