#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Feb 23 10:44:21 2021

@author: lbresin
"""
import pandas as pd

def get_number_of_lines(file):
    with open(file, 'r') as fid:
        return len(fid.readlines())
    
    
def export2hgs(file, df):
    with open(file, 'w') as fid:
        fid.write('clear chosen nodes\nclear chosen elements\nclear chosen zones\nclear chosen faces\n\n')
        for report_cell in pumping_wells.hsi_pumping_cell.unique():
            fid.write('!----- HSI REPORT ZONE: \t {}\n'.format(report_cell))
            for index in pumping_wells.index[pumping_wells.hsi_pumping_cell == report_cell]:         
                fid.write('choose node\n')
                fid.write('  ' + '{0:8.2f}'.format(pumping_wells.x_abs[index]) + '  ' + '{0:8.2f}'.format(pumping_wells.y_abs[index]) + '  ' + '{0:8.2f}'.format(pumping_wells.z_abs[index])+ '\n')
            # fid.write('choose nodes list\n')
            # fid.write('  ./hgs_files/area_selection_files/nn_pumping_report_cell_{}\n'.format(report_cell))
            fid.write('\nwrite chosen nodes\n')
            fid.write('  pumping_report_zone_{}_nodes_nn.txt\n'.format(report_cell))
            fid.write('write chosen nodes xyz\n')
            fid.write('  pumping_report_zone_{}_nodes_xyz.txt\n\n'.format(report_cell))
            fid.write('create node set\n')
            fid.write('  pumping_report_zone_{}\n\n'.format(report_cell))
            fid.write('boundary condition\n')
            fid.write('  type\n')
            fid.write('  flux nodal\n\n')
            fid.write('  node set\n')
            fid.write('  pumping_report_zone_{}\n\n'.format(report_cell))
            fid.write('  time value table\n')
            
            #node_count = get_number_of_lines('./hgs_area_selection_files/nn_pumping_report_cell_{}'.format(report_cell))
            node_count = report_cell_well_counts['{}'.format(report_cell)]
            
            for date in df.index:
                pump_rate = df.loc[date, report_cell] / node_count * -1
                sim_time = (date-simulation_start).days           
                fid.write('  {} {}\n'.format(sim_time, pump_rate))
                
            fid.write('  end\n')
            fid.write('  interpolate\n')
            fid.write('  tecplot output\n')
            fid.write('  nodal flux reduction by pressure head\n  1\n  10\n')
            fid.write('end\n')
            fid.write('\nclear chosen nodes\nclear chosen elements\nclear chosen zones\nclear chosen faces\n\n')    

if __name__ == '__main__':
    simulation_start = pd.Timestamp('1950-09-15')
    scenario_bc_start = pd.Timestamp('2019-09-15')
    scenario_bc_end = pd.Timestamp('2070-12-31')
    
    ### OPEN RELEVANT DATA ###  
    df_pump_scenario_b = pd.read_csv('./monthly_pumping_b.csv', index_col=0)  # m³/d
    df_pump_scenario_b.index = pd.to_datetime(df_pump_scenario_b.index)
    df_pump_scenario_b = df_pump_scenario_b[df_pump_scenario_b.index >= scenario_bc_start]
    df_pump_scenario_b = df_pump_scenario_b[df_pump_scenario_b.index <= scenario_bc_end]
    df_pump_scenario_b.columns = df_pump_scenario_b.columns.astype(int)
    
    df_pump_scenario_rri = pd.read_csv('./monthly_pumping_rri.csv', index_col=0)  # m³/d
    df_pump_scenario_rri.index = pd.to_datetime(df_pump_scenario_rri.index)
    df_pump_scenario_rri = df_pump_scenario_rri[df_pump_scenario_rri.index >= scenario_bc_start]
    df_pump_scenario_rri = df_pump_scenario_rri[df_pump_scenario_rri.index <= scenario_bc_end]
    df_pump_scenario_rri.columns = df_pump_scenario_rri.columns.astype(int)
    
    df_pump_scenario_rnc = pd.read_csv('./monthly_pumping_rnc.csv', index_col=0)  # m³/d
    df_pump_scenario_rnc.index = pd.to_datetime(df_pump_scenario_rnc.index)
    df_pump_scenario_rnc = df_pump_scenario_rnc[df_pump_scenario_rnc.index >= scenario_bc_start]
    df_pump_scenario_rnc = df_pump_scenario_rnc[df_pump_scenario_rnc.index <= scenario_bc_end]
    df_pump_scenario_rnc.columns = df_pump_scenario_rnc.columns.astype(int)
      
    ### open pumping well database ###
    pumping_wells = pd.read_csv('./pumping_wells.csv')
    pumping_wells = pumping_wells[~pumping_wells.hsi_pumping_cell.isna()]
    pumping_wells = pumping_wells[pumping_wells.hsi_pumping_cell != '214']
    report_cell_well_counts = dict({'210':249, '211':148, '212':58, '220':32})
    
    ### PLOT MERGED DATA FOR COMPARISON ###
    plot_hep_b = df_pump_scenario_b.copy()
    plot_hep_b = plot_hep_b.reindex(pd.date_range(start=plot_hep_b.index.min(), end=plot_hep_b.index.max())).interpolate()
    #plot_hep_b = plot_hep_b[plot_hep_b.index >=  pd.Timestamp('2019-01-01')]
    b = plot_hep_b.groupby(by=plot_hep_b.index.year).sum().sum(axis=1)
    
    
    plot_hep_rnc = df_pump_scenario_rnc.copy()
    plot_hep_rnc = plot_hep_rnc.reindex(pd.date_range(start=plot_hep_rnc.index.min(), end=plot_hep_rnc.index.max())).interpolate()
    plot_hep_rnc = plot_hep_rnc[plot_hep_rnc.index >=  pd.Timestamp('2019-01-01')]
    rnc = plot_hep_rnc.groupby(by=plot_hep_rnc.index.year).sum().sum(axis=1)
    
    plot_hep_rri = df_pump_scenario_rri.copy()
    plot_hep_rri = plot_hep_rri.reindex(pd.date_range(start=plot_hep_rri.index.min(), end=plot_hep_rri.index.max())).interpolate()
    plot_hep_rri = plot_hep_rri[plot_hep_rri.index >=  pd.Timestamp('2019-01-01')]
    rri = plot_hep_rri.groupby(by=plot_hep_rri.index.year).sum().sum(axis=1)
    
    
    #plot
    
    import matplotlib.pyplot as plt
   
    fig = plt.figure(figsize=(10/2.5,5.0/2.5))
    ax = plt.gca()
    
    ax.plot(b/1e6, label='Baseline', c=(252/255,141/255,89/255), linestyle = ':')
    ax.plot(rri/1e6, label='RRI', c=(145/255,191/255,219/255))
    ax.plot(rnc/1e6, label='RNC', c=(153/255,213/255,148/255))
    plt.fill_between(rnc.index, rnc/1e6, rri/1e6, alpha = 0.3, color='grey')
    ax.set_ylabel('Well abstraction (MCM/A)')
    ax.set_xlim([2019, 2070])
    plt.legend(fancybox=True, shadow=True, ncol=3, frameon=False, fontsize=9)
    plt.savefig('annual_pump_mcm.png')
    


    ### CREATE HGS INPUT INSTRUCTIONS ###
    export2hgs('./export/pumping_rate_b.in', df_pump_scenario_b)
    export2hgs('./export/pumping_rate_rnc.in', df_pump_scenario_rnc)
    export2hgs('./export/pumping_rate_rri.in', df_pump_scenario_rri)
