#!/usr/bin/env python3
# -*- coding: utf-8 -*-
''' \
USAGE:  python script.py -i1 cpu_core_num
    -i1  : input1 number of cpu cores
'''

import sys 
import os
from termcolor import cprint

# # Change current working directory for testing
# if input('Changing working directory (y:yes, n: no): '):
#     os.chdir("/home/lbresin/ownCloud/MedWater/hydrogeosphere/westernmountainaquifer_33_varsat_dcpe_test_zoned_rch")

in_arr = sys.argv
if '-i1' not in in_arr:
    print (__doc__)
    raise NameError('error: input options are not provided')
else:
    cpu_num = in_arr[in_arr.index('-i1') + 1]

file='parallelindx.dat'


### Alter existing parallelindx.dat, when there is a RESTART_FILE_INFO.dat. This comes in handy when a exisiting simulation run was interrupted and needs to be continued ###
if os.path.isfile('RESTART_FILE_INFO.dat') & os.path.isfile('parallelindx.dat'):
    ## Get parallelindx
    with open(file, 'r') as fid:
        parallelindx = fid.readlines()
    
    ## Adjust Simulation Parameters
    # Number_of_CPU
    parallelindx[1] = '\t{}\n'.format(cpu_num)
    # Num_Domain_Partitiong
    parallelindx[3] = '\t{}\n'.format(cpu_num)
    # Solver_Type
    if cpu_num =='1':
        parallelindx[5] = '\t1\n'
    else:
        parallelindx[5] = '\t2\n'
    
    with open(file, 'w') as fid:
        fid.write(''.join(parallelindx))

### Override any existing parallelindx.dat ###
else:
    with open(file, 'w') as fid:
        fid.write('__Number_of_CPU\n')
        fid.write('\t'+str(cpu_num)+'\n')
        fid.write('__Num_Domain_Partitiong\n')
        fid.write('\t'+str(cpu_num)+'\n')
        fid.write('__Solver_Type\n')
        if cpu_num=='1':
            fid.write('\t1\n')
        else:
            fid.write('\t2\n')
        fid.write('__Coloring_Input\n\tF\n__Wrting_Output_Time\n\t-1.00000000000000\n__Simulation_Restart\n\t1')
                
    with open(file, 'r') as fid:
        parallelindx = fid.readlines()
  
print('successfully created parallelindx.dat!\n\n')
cprint('---- parallelindx.dat: \n'+''.join(parallelindx)+'\n\n', 'yellow')