#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Feb 20 13:29:41 2020

@author: lbresin
"""

import pandas as pd
import linecache
import numpy as np
from matplotlib import pyplot as plt
import datetime as dt
import matplotlib.dates as mdates

def read_water_balance(file):
    # get columns
    import linecache
    import pyparsing as pp
    
    # Get column names
    col = linecache.getline(file, 2).lstrip('VARIABLES').lstrip(' ').lstrip('=').rstrip(',\n')
    sep = pp.pyparsing_common.comma_separated_list.copy().addParseAction(pp.tokenMap(lambda s: s.strip('"')))
    col = sep.parseString(col).asList()

    #  Read Water balance file
    df_water_balance = pd.read_csv(file, skiprows=3, delim_whitespace=True, header=None)
    df_water_balance.columns = col

    return df_water_balance

### Change current working directory for testing ###
import os
if input('Changing working directory (y:yes, n: no): '):
    os.chdir("/home/lbresin/hydrogeosphere/westernmountainaquifer_40_varsat_dcpe_measure_storage_vadose_zone_soil_balance_exch_flux")

### Gather list of all boundary conditions from ${Prefix}o.boundary_conditions file ###
prefix =  linecache.getline('batch.pfx',1).rstrip("\n")
file_bc = prefix + 'o.boundary_conditions'
with open(file_bc, 'r') as file_open:
    bc_name, bc_type, bc_id, bc_set_name, bc_domain = [], [], [], [], []

    for line in file_open.readlines():
        if '! name' in line:
            bc_name.append(line.replace(' ','').split('!')[0])
        elif '! id' in line:
            bc_id.append(line.replace(' ','').split('!')[0])
        elif '! type' in line:
            bc_type.append(line.replace(' ','').split('!')[0])
        elif '! set name' in line:
            bc_set_name.append(line.replace(' ','').split('!')[0])
        elif '! domain' in line:
            bc_domain.append(line.replace(' ','').split('!')[0])
        else:
            pass
     
    data_bc = pd.DataFrame(data = {'name': bc_name, 'set_name': bc_set_name, 'domain': bc_domain, 'type': bc_type}, index=np.array(bc_id, dtype=int))
data_bc.domain = data_bc.domain.astype(int)
data_bc.type = data_bc.type.astype(int)


### Gather flux data from ${prefix}o.lst file ###
# file = prefix + 'o.lst'
# u_list = list()
# i_list = list()
# with open(file, 'r') as file_open:
#     lines = file_open.readlines()
#     for i, line in enumerate(lines):
#         if 'Boundary condition name' not in line:
#             pass
#         else:
#             u_list.append(list(range(i+2, i+2+len(data_bc))))
#             i_list.append(i-2)
# u_list = np.hstack(u_list)

# tmp = dict(zip(bc_name, [list() for i in range(len(bc_name))]))
# for u in u_list:
#     line = linecache.getline(file,u).split()
#     tmp[line[0]].append(float(line[-2]))
# data_bc['flux'] = [i[1] for i in list(tmp.items())]

# tmp = list()
# for i in i_list:
#     tmp.append(float(linecache.getline(file,i).split()[-1]))
# data_bc['time'] = [tmp for i in range(len(data_bc))]

### Gather flux data from ${prefix}o.lst file ###
file = '{}o.water_balance.dat'.format(prefix)
df_water_balance = read_water_balance(file)
# Add date to df
reference_date = pd.Timestamp('1950-09-15')
df_water_balance.loc[:,'date'] = df_water_balance.Time.apply(lambda x: reference_date + pd.Timedelta(value=x, unit='days'))
df_water_balance = df_water_balance.set_index('date')
df_water_balance = df_water_balance.drop('Time', axis=1)

### RESAMPLE TO ANNUAL VALUE ###
delta_t = list()
for i_num, i_str in enumerate(df_water_balance.index):
    if i_num > 0:
        delta_t.append((i_str-df_water_balance.index[i_num-1]).total_seconds() / (24*60*60)) # days
    else:
        delta_t.append((i_str-reference_date).total_seconds() / (24*60*60))
delta_t = np.array(delta_t)

df_water_balance_annual = df_water_balance.loc[:,data_bc.name]
for key in df_water_balance_annual.keys():
    df_water_balance_annual.loc[:, key] *= delta_t
df_water_balance_annual = df_water_balance_annual.groupby(by=df_water_balance_annual.index.year).sum()


### Plot fluxes
def export_annual_aggregated__water_balance():
    df_export = pd.DataFrame(columns=(['deep_percolation', 'salt_water_intrusion', 'well_injection', 'spring_discharge', 'well_abstraction']))
    # Percolation
    keys_perc = data_bc.name[data_bc.name.str.startswith('Rain')]
    df_export['deep_percolation'] = df_water_balance_annual.loc[:,keys_perc].sum(axis=1)
    # Salt Water
    keys_intr =  data_bc.name[data_bc.set_name.str.startswith('salt_water_intrusion')]
    df_export['salt_water_intrusion'] = df_water_balance_annual.loc[:,keys_intr].sum(axis=1)
    # Well Injection
    keys_inj =  data_bc.name[data_bc.set_name.str.startswith('injection')]
    df_export['well_injection'] = df_water_balance_annual.loc[:,keys_inj].sum(axis=1)
    # Spring Discharge
    keys_drain =  data_bc.name[data_bc.name.str.startswith('SimpleDrain')]
    df_export['spring_discharge'] = df_water_balance_annual.loc[:,keys_drain].sum(axis=1)
    # Well Abstraction
    keys_abs =  data_bc.name[data_bc.set_name.str.startswith('pumping')]
    df_export['well_abstraction'] = df_water_balance_annual.loc[:,keys_abs].sum(axis=1)
    # Difference
    df_export['difference'] = df_water_balance_annual.loc[:,keys_abs.to_list() + keys_perc.to_list() + keys_intr.to_list() + keys_inj.to_list()+ keys_drain.to_list()].sum(axis=1)
    
    df_export.to_csv('./plot/'+prefix+'_water_balance.csv')
export_annual_aggregated__water_balance()
    
def plot_complete_water_balance():
    fig, ax  = plt.subplots(figsize = [18/2.5, 10/2.5], dpi=300)
    
    # Percolation
    keys_perc = data_bc.name[data_bc.name.str.startswith('Rain')]
    plt.bar(df_water_balance_annual.index, df_water_balance_annual.loc[:,keys_perc].sum(axis=1), 0.9, color= '#4575b4', label='Percolation')
    
    # Salt Water
    keys_intr =  data_bc.name[data_bc.set_name.str.startswith('salt_water_intrusion')]
    plt.bar(df_water_balance_annual.index, df_water_balance_annual.loc[:,keys_intr].sum(axis=1), 0.9, bottom=df_water_balance_annual.loc[:,keys_perc].sum(axis=1), color= '#542788', label='Salt Water Intrusion')
    
    # Well Injection
    keys_inj =  data_bc.name[data_bc.set_name.str.startswith('injection')]
    plt.bar(df_water_balance_annual.index, df_water_balance_annual.loc[:,keys_inj].sum(axis=1), 0.9, bottom=df_water_balance_annual.loc[:,keys_perc].sum(axis=1)+df_water_balance_annual.loc[:,keys_intr].sum(axis=1), color= '#91cf60', label='Well Injection')
    
    # Spring Discharge
    keys_drain =  data_bc.name[data_bc.name.str.startswith('SimpleDrain')]
    plt.bar(df_water_balance_annual.index, df_water_balance_annual.loc[:,keys_drain].sum(axis=1), 0.9, color= '#fc8d59', label='Spring Discharge')
    
    # Well Abstraction
    keys_abs =  data_bc.name[data_bc.set_name.str.startswith('pumping')]
    plt.bar(df_water_balance_annual.index, df_water_balance_annual.loc[:,keys_abs].sum(axis=1), 0.9, bottom=df_water_balance_annual.loc[:,keys_drain].sum(axis=1), color= '#d73027', label='Well Pumping')
    
    plt.plot(df_water_balance_annual.loc[:,keys_abs.to_list() + keys_perc.to_list() + keys_intr.to_list() + keys_inj.to_list()+ keys_drain.to_list()].sum(axis=1), color='black', linestyle=':', label='Difference')
    
    plt.xlabel('Year')
    plt.ylabel('Volumetric flux (m³/year)')
    
    plt.xlim([df_water_balance_annual.index.min()-0.9/2, df_water_balance_annual.index.max()+0.9/2])
    
    plt.legend(fancybox=None, frameon=False, ncol=2, fontsize=8)
    plt.tight_layout()
    fig.savefig('./plot/'+prefix+'_water_balance.eps', format='eps', dpi=1000)
    fig.savefig('./plot/'+prefix+'_water_balance.png', format='png', dpi=1000)
    fig.savefig('./plot/'+prefix+'_water_balance.pdf', format='pdf', dpi=1000)
plot_complete_water_balance()

def plot_bc_seperatly():
       
    # Percolation
    fig, ax  = plt.subplots(figsize = [18/2.5, 10/2.5], dpi=300)
    keys_perc = data_bc.name[data_bc.name.str.startswith('Rain')]
    plt.bar(df_water_balance_annual.index, df_water_balance_annual.loc[:,keys_perc].sum(axis=1), 0.9, color= '#4575b4', label='Percolation')
    
    plt.xlabel('Year')
    plt.ylabel('Volumetric flux (m³/year)')
    
    plt.xlim([df_water_balance_annual.index.min()-0.9/2, df_water_balance_annual.index.max()+0.9/2])
    
    plt.legend(fancybox=None, frameon=False, ncol=2, fontsize=8)
    plt.tight_layout()
    fig.savefig('./plot/'+prefix+'_water_balance_percolation.eps', format='eps', dpi=1000)
    fig.savefig('./plot/'+prefix+'_water_balance_percolation.png', format='png', dpi=1000)
    fig.savefig('./plot/'+prefix+'_water_balance_percolation.pdf', format='pdf', dpi=1000)  
    plt.close()
    
    
    # Salt Water
    fig, ax  = plt.subplots(figsize = [18/2.5, 10/2.5], dpi=300)
    
    keys_intr =  data_bc.name[data_bc.set_name.str.startswith('salt_water_intrusion')]
    plt.bar(df_water_balance_annual.index, df_water_balance_annual.loc[:,keys_intr].sum(axis=1), 0.9, color= '#542788', label='Salt Water Intrusion')
    
    plt.xlabel('Year')
    plt.ylabel('Volumetric flux (m³/year)')
    
    plt.xlim([df_water_balance_annual.index.min()-0.9/2, df_water_balance_annual.index.max()+0.9/2])
    
    plt.legend(fancybox=None, frameon=False, ncol=2, fontsize=8)
    plt.tight_layout()
    fig.savefig('./plot/'+prefix+'_water_balance_salt_water.eps', format='eps', dpi=1000)
    fig.savefig('./plot/'+prefix+'_water_balance_salt_water.png', format='png', dpi=1000)
    fig.savefig('./plot/'+prefix+'_water_balance_salt_water.pdf', format='pdf', dpi=1000)  
    plt.close()
    
    # Well Injection
    fig, ax  = plt.subplots(figsize = [18/2.5, 10/2.5], dpi=300)
    
    keys_inj =  data_bc.name[data_bc.set_name.str.startswith('injection')]
    plt.bar(df_water_balance_annual.index, df_water_balance_annual.loc[:,keys_inj].sum(axis=1), 0.9,  color= '#91cf60', label='Well Injection')
    
    plt.xlabel('Year')
    plt.ylabel('Volumetric flux (m³/year)')
    
    plt.xlim([df_water_balance_annual.index.min()-0.9/2, df_water_balance_annual.index.max()+0.9/2])
    
    plt.legend(fancybox=None, frameon=False, ncol=2, fontsize=8)
    plt.tight_layout()
    fig.savefig('./plot/'+prefix+'_water_balance_injection.eps', format='eps', dpi=1000)
    fig.savefig('./plot/'+prefix+'_water_balance_injection.png', format='png', dpi=1000)
    fig.savefig('./plot/'+prefix+'_water_balance_injection.pdf', format='pdf', dpi=1000)  
    plt.close()
    
    # Spring Discharge
    fig, ax  = plt.subplots(figsize = [18/2.5, 10/2.5], dpi=300)

    keys_drain =  data_bc.name[data_bc.name.str.startswith('SimpleDrain')]
    plt.bar(df_water_balance_annual.index, df_water_balance_annual.loc[:,keys_drain].sum(axis=1), 0.9, color= '#fc8d59', label='Spring Discharge')

    plt.xlabel('Year')
    plt.ylabel('Volumetric flux (m³/year)')
    
    plt.xlim([df_water_balance_annual.index.min()-0.9/2, df_water_balance_annual.index.max()+0.9/2])
    
    plt.legend(fancybox=None, frameon=False, ncol=2, fontsize=8)
    plt.tight_layout()
    fig.savefig('./plot/'+prefix+'_water_balance_spring_discharge.eps', format='eps', dpi=1000)
    fig.savefig('./plot/'+prefix+'_water_balance_spring_discharge.png', format='png', dpi=1000)
    fig.savefig('./plot/'+prefix+'_water_balance_spring_discharge.pdf', format='pdf', dpi=1000)  
    plt.close()
    
    # Well Abstraction
    fig, ax  = plt.subplots(figsize = [18/2.5, 10/2.5], dpi=300)
    
    keys_abs =  data_bc.name[data_bc.set_name.str.startswith('pumping')]
    plt.bar(df_water_balance_annual.index, df_water_balance_annual.loc[:,keys_abs].sum(axis=1), 0.9,  color= '#d73027', label='Well Pumping')
       
    plt.xlabel('Year')
    plt.ylabel('Volumetric flux (m³/year)')
    
    plt.xlim([df_water_balance_annual.index.min()-0.9/2, df_water_balance_annual.index.max()+0.9/2])
    
    plt.legend(fancybox=None, frameon=False, ncol=2, fontsize=8)
    plt.tight_layout()
    fig.savefig('./plot/'+prefix+'_water_balance_pumping.eps', format='eps', dpi=1000)
    fig.savefig('./plot/'+prefix+'_water_balance_pumping.png', format='png', dpi=1000)
    fig.savefig('./plot/'+prefix+'_water_balance_pumping.pdf', format='pdf', dpi=1000)    
plot_bc_seperatly()    
    
    