#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Jan  6 10:27:21 2021

@author: lbresin
"""
import os 
from termcolor import cprint
import shutil
import glob
import pandas as pd

def open_dat_file(file_dir):
    '''
    This opens a .dat files and stores its content to the output
    ''' 
    with open(file_dir, 'r') as file:
        lines=file.readlines()[1:]
        keys=lines[0].replace('VARIABLES','').replace('=','').lstrip(' ').replace(',','').replace('\n','').replace('  ',' ').replace('  ',' ').replace('  ',' ').replace('" "','$break$').replace('""','$break$').lstrip('"').rstrip('"').split('$break$')
        data=map(str.split,lines[2:])
        data=pd.DataFrame(data=list(data),columns=keys, dtype=float)
    return data

def get_target_times_from_eco(file_dir):
    with open(file_dir, 'r') as fid:
        lines = fid.readlines()

        for i in reversed(range(len(lines))):
            if 'Target times' in lines[i]:
                start = i + 3
        for i in range(start, len(lines)):
            if '-------------------------------------' in lines[i]:
                end = i
        target_times = pd.DataFrame(columns=['step', 'timestep', 'time'], data=map(lambda x: x.split(),lines[start:end])).set_index('step')
        target_times.index = target_times.index.astype(int)
        
        return target_times.astype(float)
    
def get_output_times_from_eco(file_dir):
    with open(file_dir, 'r') as fid:
        lines = fid.readlines()

        for i in reversed(range(len(lines))):
            if 'EXIT OUTPUT TIME' in lines[i]:
                end = i - 1
        for i in reversed(range(end)):
            if 'output times' in lines[i]:
                start = i + 1
        output_times = pd.DataFrame(columns=['time'], data=map(lambda x: x.split()[-1],lines[start:end]))
        output_times.index = output_times.index.astype(int) + 1
        
        return output_times.astype(float)
    
def get_simulated_output_times_from_newton_info(file_dir):
    with open(file_dir, 'r') as fid:
        lines = fid.readlines()
        last_time = dict(zip(lines[1].replace('VARIABLES','').replace('=','').lstrip(' ').replace(',','').replace('\n','').replace('  ',' ').replace('  ',' ').replace('  ',' ').replace('" "','$break$').replace('""','$break$').lstrip('"').rstrip('"').split('$break$'),
                        map(float,lines[-1].split())))['Time']
        
        return output_times.loc[output_times['time']<=last_time, 'time']
    
def get_intial_timestep_from_newton_info(file_dir):
    df = open_dat_file(file_dir)
    return df.loc[df['Time'] == initial_time, 'Time step'].values[0]    
        
        
cprint('This Script adjusts the files "RESTART_FILE_INFO.dat" and “parallelindx.dat” to allow for the continuation of an interrupted simulation.  The simulation continues at the last successfully computed timestep. All "*.dat" files are backed up and may be merge later with the files of the second run.', 'green')

# ## Change current working directory for testing ###
# if input('Changing working directory (y:yes, n: no): ') == 'y':
#     os.chdir("/home/lbresin/hydrogeosphere/westernmountainaquifer_40_varsat_dcpe_climate_scenario_rnc_run1")

### Get some relevant infos ###

# current working dir
pwd = os.path.realpath('./')

# simulation name
sim_name = pwd.split('/')[-1]

# model prefix
with open('batch.pfx', 'r') as batch_file:
    model_prefix = batch_file.readline().rstrip('\n')
    
# model data
target_times = get_target_times_from_eco('{}o.eco'.format(model_prefix))
output_times = get_output_times_from_eco('{}o.eco'.format(model_prefix))

# last timestep and time of previous simulation run
sim_output_times = get_simulated_output_times_from_newton_info('{}o.newton_info.dat'.format(model_prefix))

# Extract the Restart info
initial_time = sim_output_times.iloc[-1] # last absolute time
ntloop_target = target_times[target_times['time'] > initial_time].index[0] # next target time step
iphead = sim_output_times[sim_output_times == initial_time].index[0] # last successful output time step
initial_timestep = get_intial_timestep_from_newton_info('{}o.newton_info.dat'.format(model_prefix))

### Modify “RESTART_FILE_INFO.dat” ###
"""
Explanation of the variables:
__initial_time: 	Absolute time that has been successfully computed and from where the simulation shall be continued
__initial_timestep	Initial Time step to continue with
__ntloop_target	Next Time step number, This can be read from the *.eco file
__iphead		Time step number that has been successfully computed and from where the simulation shall be continued. The file {PREFIX}o.head_{pm\olf\dual}.				{iphead} will be used as an initial head to continue the simulation

"""
with open('RESTART_FILE_INFO.dat', 'w') as fid:
    fid.write('__initial_time\n  {}\n'.format(initial_time))
    fid.write('__initial_timestep\n  {}\n'.format(initial_timestep))
    fid.write('__ntloop_target\n  {}\n'.format(ntloop_target))
    fid.write('__iphead\n  {}\n'.format(iphead))

### Modify “parallelindx.dat” ###
with open('parallelindx.dat', 'r') as fid:
    parallelindx = fid.readlines()
parallelindx[11] = '\t{}'.format(iphead)
with open('parallelindx.dat', 'w') as fid:
    fid.write(''.join(parallelindx))
    
    
### Print Modifcations ###
with open('RESTART_FILE_INFO.dat', 'r') as fid:
    print('\n--- RESTART_FILE_INFO.dat: \n'+fid.read())
with open('parallelindx.dat', 'r') as fid:
    print('--- parallelindx.dat: \n'+fid.read()+'\n')

cprint('WARNING: Continue the simulation with “hgs.x”. (Do not run “grok.x” again!)', 'red')